@@ -16,7 +16,7 @@ The following attributes are added to each element:
        interpolated from sftable.
 
    :func:`Xray.f0`
-       Returns f0 for the given vector Q, with Q[i] in $[0,24\pi]$ |1/Ang|.
+       Returns f0 for the given vector Q, with Q[i] in $[0, 24\pi]$ |1/Ang|.
 
    :func:`Xray.sld`
        Returns scattering length density (*real*, *imaginary*) for the
@@ -74,7 +74,7 @@ Emission line tables
 ====================
 
 Data for the $K_\alpha$ and $K_\beta$ lines comes from
-[#Deslattes2003], with the full tables available at
+[#Deslattes2003]_, with the full tables available at
 `<http://www.nist.gov/pml/data/xraytrans/index.cfm>`_.
 Experimental Values are used, truncated to 4 digits
 of precision to correspond to the values for the subset
@@ -141,26 +141,31 @@ with minor formatting changes:
 
   .. table::
 
-           ========  ==========   =================
-           Element   Updated      Energy Range (eV)
-           ========  ==========   =================
-           Mg        Jan 2011     10-1300
-           Zr        Apr 2010     20-1000
-           La        Jun 2007     14-440
-           Gd        Jun 2007     12-450
-           Sc        Apr 2006     50-1300
-           Ti        Aug 2004     20-150
-           Ru        Aug 2004     40-1300
-           W         Aug 2004     35-250
-           Mo        Aug 2004     25-60
-           Be        Aug 2004     40-250
-           Mo        Nov 1997     10-930
-           Fe        Oct 1995     600-800
-           Si        Jun 1995     30-500
-           Au        Jul 1994     2000-6500
-           Mg,Al,Si  Jan 1994     30-200
-           Li        Nov 1994     2000-30000
-           ========  ==========   =================
+    ========  ==========   =================
+    Element   Updated      Energy Range (eV)
+    ========  ==========   =================
+    Pt        Aug 2023     0.1-800
+    Cr        Aug 2023     0.4-813
+    Nb        Aug 2023     10-1000
+    Y         Aug 2023     50-1300
+    Er        Aug 2023     3-1580
+    Mg        Jan 2011     10-1300
+    Zr        Apr 2010     20-1000
+    La        Jun 2007     14-440
+    Gd        Jun 2007     12-450
+    Sc        Apr 2006     50-1300
+    Ti        Aug 2004     20-150
+    Ru        Aug 2004     40-1300
+    W         Aug 2004     35-250
+    Mo        Aug 2004     25-60
+    Be        Aug 2004     40-250
+    Mo        Nov 1997     10-930
+    Fe        Oct 1995     600-800
+    Si        Jun 1995     30-500
+    Au        Jul 1994     2000-6500
+    Mg,Al,Si  Jan 1994     30-200
+    Li        Nov 1994     2000-30000
+    ========  ==========   =================
 
 
   Data available at:
@@ -170,30 +175,27 @@ with minor formatting changes:
        photoabsorption, scattering, transmission, and reflection at E=50-30000 eV,
        Z=1-92", Atomic Data and Nuclear Data Tables 54 no.2, 181-342 (July 1993).
 
-.. [#Deslattes2003] R. D. Deslattes, E. G. Kessler, Jr., P. Indelicato, L. de Billy, 
+.. [#Deslattes2003] R. D. Deslattes, E. G. Kessler, Jr., P. Indelicato, L. de Billy,
        E. Lindroth, and J. Anton.  Rev. Mod. Phys. 75, 35-99 (2003).
-
 """
-from __future__ import with_statement
 __all__ = ['Xray', 'init', 'init_spectral_lines',
-           'xray_energy','xray_wavelength',
-           'xray_sld','xray_sld_from_atoms',
-           'emission_table','sld_table','plot_xsf',
+           'xray_energy', 'xray_wavelength',
+           'xray_sld', 'xray_sld_from_atoms',
+           'emission_table', 'sld_table', 'plot_xsf',
            'index_of_refraction', 'mirror_reflectivity',
            ]
 import os.path
-import glob
 
-import numpy
+import numpy as np
 from numpy import nan, pi, exp, sin, cos, sqrt, radians
 
 from .core import Element, Ion, default_table, get_data_path
-from .constants import (avogadro_number, plancks_constant, speed_of_light,
-                        electron_radius)
-from .util import require_keywords
+from .constants import (
+    avogadro_number, planck_constant, speed_of_light, electron_volt,
+    electron_radius)
 
 def xray_wavelength(energy):
-    """
+    r"""
     Convert X-ray energy to wavelength.
 
     :Parameters:
@@ -210,14 +212,14 @@ def xray_wavelength(energy):
 
     where:
 
-        $h$ = planck's constant in eV\ |cdot|\ s
+        $h$ = Planck constant in J\ |cdot|\ s
 
         $c$ = speed of light in m/s
     """
-    return plancks_constant*speed_of_light/numpy.asarray(energy)*1e7
+    return planck_constant/electron_volt*speed_of_light/np.asarray(energy)*1e7
 
 def xray_energy(wavelength):
-    """
+    r"""
     Convert X-ray wavelength to energy.
 
     :Parameters:
@@ -234,21 +236,22 @@ def xray_energy(wavelength):
 
     where:
 
-        $h$ = planck's constant in eV\ |cdot|\ s
+        $h$ = Planck constant in J\ |cdot|\ s
 
         $c$ = speed of light in m/s
     """
-    return plancks_constant*speed_of_light/numpy.asarray(wavelength)*1e7
+    return planck_constant/electron_volt*speed_of_light/np.asarray(wavelength)*1e7
 
-class Xray(object):
+class Xray:
     """
     X-ray scattering properties for the elements. Refer help(periodictable.xsf)
     from command prompt for details.
     """
+    sftable_units: tuple[str, str, str] = ("eV", "", "")
+    scattering_factors_units: tuple[str, str] = ("", "")
+    sld_units: tuple[str, str] = ("1e-6/Ang^2", "1e-6/Ang^2")
+    element: Element
     _nff_path = get_data_path('xsf')
-    sftable_units = ["eV","",""]
-    scattering_factors_units = ["",""]
-    sld_units = ["1e-6/Ang^2","1e-6/Ang^2"]
     _table = None
     def __init__(self, element):
         self.element = element
@@ -261,17 +264,16 @@ class Xray(object):
             filename = os.path.join(self._nff_path,
                                     self.element.symbol.lower()+".nff")
             if self.element.symbol != 'n' and os.path.exists(filename):
-                xsf = numpy.loadtxt(filename,skiprows=1).T
-                xsf[1,xsf[1]==-9999.] = numpy.NaN
+                xsf = np.loadtxt(filename, skiprows=1).T
+                xsf[1, xsf[1] == -9999.] = np.nan
                 xsf[0] *= 0.001  # Use keV in table rather than eV
                 self._table = xsf
         return self._table
-    sftable = property(_gettable,doc="X-ray scattering factor table (E,f1,f2)")
+    sftable = property(_gettable, doc="X-ray scattering factor table (E,f1,f2)")
 
-    @require_keywords
-    def scattering_factors(self, energy=None, wavelength=None):
+    def scattering_factors(self, *, energy=None, wavelength=None):
         """
-        X-ray scattering factors f',f''.
+        X-ray scattering factors f', f''.
 
         :Parameters:
             *energy* : float or vector | keV
@@ -287,21 +289,23 @@ class Xray(object):
         """
         xsf = self.sftable
         if xsf is None:
-            return None,None
+            return None, None
 
         if wavelength is not None:
             energy = xray_energy(wavelength)
         if energy is None:
             raise TypeError('X-ray scattering factors need wavelength or energy')
 
-        scalar = numpy.isscalar(energy)
+        scalar = np.isscalar(energy)
         if scalar:
-            energy = numpy.array([energy])
-        f1 = numpy.interp(energy,xsf[0],xsf[1],left=nan,right=nan)
-        f2 = numpy.interp(energy,xsf[0],xsf[2],left=nan,right=nan)
+            energy = np.array([energy])
+        f1 = np.interp(energy, xsf[0], xsf[1], left=nan, right=nan)
+        f2 = np.exp(np.interp(
+            np.log(energy), np.log(xsf[0]), np.log(xsf[2]),
+            left=nan, right=nan))
         if scalar:
-            f1,f2 = f1[0],f2[0]
-        return f1,f2
+            f1, f2 = f1[0], f2[0]
+        return f1, f2
 
     def f0(self, Q):
         r"""
@@ -332,8 +336,7 @@ class Xray(object):
                                 charge=self.element.charge)
         return f
 
-    @require_keywords
-    def sld(self, wavelength=None, energy=None):
+    def sld(self, *, wavelength=None, energy=None):
         r"""
         X ray scattering length density.
 
@@ -369,16 +372,15 @@ class Xray(object):
         Data comes from the Henke Xray scattering factors database at the
         Lawrence Berkeley Laboratory Center for X-ray Optics.
         """
-        f1,f2 = self.scattering_factors(wavelength=wavelength, energy=energy)
+        f1, f2 = self.scattering_factors(wavelength=wavelength, energy=energy)
         if f1 is None or self.element.number_density is None:
-            return None,None
+            return None, None
         rho = f1*electron_radius*self.element.number_density*1e-8
         irho = f2*electron_radius*self.element.number_density*1e-8
-        return rho,irho
+        return rho, irho
 
 # Note: docs and function prototype are reproduced in __init__
-@require_keywords
-def xray_sld(compound, density=None,  natural_density=None,
+def xray_sld(compound, *, density=None, natural_density=None,
              wavelength=None, energy=None):
     """
     Compute xray scattering length densities for molecules.
@@ -407,29 +409,31 @@ def xray_sld(compound, density=None,  natural_density=None,
                                 natural_density=natural_density)
     assert compound.density is not None, "scattering calculation needs density"
 
-    if wavelength is not None: energy = xray_energy(wavelength)
+    if wavelength is not None:
+        energy = xray_energy(wavelength)
     assert energy is not None, "scattering calculation needs energy or wavelength"
 
-    mass,sum_f1,sum_f2 = 0,0,0
-    for element,quantity in compound.atoms.items():
+    mass, sum_f1, sum_f2 = 0, 0, 0
+    for element, quantity in compound.atoms.items():
         mass += element.mass*quantity
-        f1,f2 = element.xray.scattering_factors(energy=energy)
-        #print element,f1,f2,wavelength
+        f1, f2 = element.xray.scattering_factors(energy=energy)
+        if f1 is None:
+            raise ValueError(f"X-ray scattering factors not available for {element}")
+        #print element, f1, f2, wavelength
         sum_f1 += f1*quantity
         sum_f2 += f2*quantity
 
     if mass == 0: # because the formula is empty
-        return 0,0
+        return 0, 0
 
     N = (compound.density/mass*avogadro_number*1e-8)
     rho = N*sum_f1*electron_radius
     irho = N*sum_f2*electron_radius
-    return rho,irho
+    return rho, irho
 
 
-@require_keywords
-def index_of_refraction(compound,density=None,natural_density=None,
-                        energy=None,wavelength=None):
+def index_of_refraction(compound, *, density=None, natural_density=None,
+                        energy=None, wavelength=None):
     """
     Calculates the index of refraction for a given compound
 
@@ -454,15 +458,90 @@ def index_of_refraction(compound,density=None,natural_density=None,
     Formula taken from http://xdb.lbl.gov (section 1.7) and checked
     against http://henke.lbl.gov/optical_constants/getdb2.html
     """
-    if energy is not None: wavelength = xray_wavelength(energy)
+    if energy is not None:
+        wavelength = xray_wavelength(energy)
     assert wavelength is not None, "scattering calculation needs energy or wavelength"
-    f1,f2 = xray_sld(compound,
-                     density=density, natural_density=natural_density,
-                     wavelength=wavelength)
+    f1, f2 = xray_sld(compound,
+                      density=density, natural_density=natural_density,
+                      wavelength=wavelength)
     return 1 - wavelength**2/(2*pi)*(f1 + f2*1j)*1e-6
 
-@require_keywords
-def mirror_reflectivity(compound, density=None, natural_density=None,
+def delta(compound, *, density=None, natural_density=None,
+                        energy=None, wavelength=None):
+    """
+    Calculates the δ component of the index of refraction for a given compound
+
+    :Parameters:
+        *compound* : Formula initializer
+            Chemical formula.
+        *density* : float | |g/cm^3|
+            Mass density of the compound, or None for default.
+        *natural_density* : float | |g/cm^3|
+            Mass density of the compound at naturally occurring isotope abundance.
+        *wavelength* : float or vector | |Ang|
+            Wavelength of the X-ray.
+        *energy* : float or vector | keV
+            Energy of the X-ray, if *wavelength* is not specified.
+
+    :Returns:
+        *delta* : float or vector | unitless
+            δ component of the index of refraction of the material at the given energy
+
+    :Notes:
+
+    Formula taken from http://xdb.lbl.gov (section 1.7) and checked
+    against http://henke.lbl.gov/optical_constants/getdb2.html.
+    """
+    if energy is not None:
+        wavelength = xray_wavelength(energy)
+    assert wavelength is not None, "scattering calculation needs energy or wavelength"
+    f1, f2 = xray_sld(compound,
+                      density=density, natural_density=natural_density,
+                      wavelength=wavelength)
+    return wavelength**2/(2*pi)*(f1)*1e-6
+
+def beta(compound, *, density=None, natural_density=None,
+                        energy=None, wavelength=None):
+    r"""
+    Calculates the β component of the index of refraction for a given compound
+
+    :Parameters:
+        *compound* : Formula initializer
+            Chemical formula.
+        *density* : float | |g/cm^3|
+            Mass density of the compound, or None for default.
+        *natural_density* : float | |g/cm^3|
+            Mass density of the compound at naturally occurring isotope abundance.
+        *wavelength* : float or vector | |Ang|
+            Wavelength of the X-ray.
+        *energy* : float or vector | keV
+            Energy of the X-ray, if *wavelength* is not specified.
+
+    :Returns:
+        *beta* : float or vector | unitless
+            β component of the index of refraction of the material at the given energy
+
+    :Notes:
+
+    Formula taken from the equation
+
+    .. math::
+
+        n = 1 - \delta + i \beta
+
+    derivative of the formula http://xdb.lbl.gov (section 1.7) using a more common
+    sign convention for the imaginary part of the index of refraction. Checked
+    against http://henke.lbl.gov/optical_constants/getdb2.html.
+    """
+    if energy is not None:
+        wavelength = xray_wavelength(energy)
+    assert wavelength is not None, "scattering calculation needs energy or wavelength"
+    f1, f2 = xray_sld(compound,
+                      density=density, natural_density=natural_density,
+                      wavelength=wavelength)
+    return wavelength**2/(2*pi)*(f2)*1e-6
+
+def mirror_reflectivity(compound, *, density=None, natural_density=None,
                         energy=None, wavelength=None,
                         angle=None, roughness=0):
     """
@@ -486,23 +565,27 @@ def mirror_reflectivity(compound, density=None, natural_density=None,
 
     :Returns:
         *reflectivity* : matrix
-            matrix of reflectivity as function of (angle,energy)
+            matrix of reflectivity as function of (angle, energy)
 
     :Notes:
 
     Formula taken from http://xdb.lbl.gov (section 4.2) and checked
     against http://henke.lbl.gov/optical_constants/mirror2.html
     """
-    if energy is not None: wavelength = xray_wavelength(energy)
+    if energy is not None:
+        wavelength = xray_wavelength(energy)
     assert wavelength is not None, "scattering calculation needs energy or wavelength"
+    assert angle is not None, "scattering calculation needs incident angle"
     angle = radians(angle)
-    if (numpy.isscalar(wavelength)): wavelength=numpy.array( [wavelength] )
-    if (numpy.isscalar(angle))     : angle =numpy.array( [angle] )
+    if np.isscalar(wavelength):
+        wavelength = np.array([wavelength])
+    if np.isscalar(angle):
+        angle = np.array([angle])
     nv = index_of_refraction(compound=compound,
                              density=density, natural_density=natural_density,
                              wavelength=wavelength)
-    ki = 2*pi/wavelength[None,:] * sin(angle[:,None])
-    kf = 2*pi/wavelength[None,:] * sqrt(nv[None,:]**2 - cos(angle[:,None])**2)
+    ki = 2*pi/wavelength[None, :] * sin(angle[:, None])
+    kf = 2*pi/wavelength[None, :] * sqrt(nv[None, :]**2 - cos(angle[:, None])**2)
     r = (ki-kf)/(ki+kf)*exp(-2*ki*kf*roughness**2)
     return abs(r)**2
 
@@ -511,7 +594,7 @@ def xray_sld_from_atoms(*args, **kw):
     """
     .. deprecated:: 0.91
 
-        :func:`xray_sld` now accepts a dictionary of \{atom\: count\} directly.
+        :func:`xray_sld` now accepts a dictionary of *{atom: count}* directly.
     """
     return xray_sld(*args, **kw)
 
@@ -617,18 +700,19 @@ def init_spectral_lines(table):
     Element.K_alpha_units = "angstrom"
     Element.K_beta1_units = "angstrom"
     for row in spectral_lines_data.split('\n'):
-        el,K_alpha,K_beta1 = row.split()
-        el = table.symbol(el) 
+        el, K_alpha, K_beta1 = row.split()
+        el = table.symbol(el)
         el.K_alpha = float(K_alpha)
         el.K_beta1 = float(K_beta1)
 
 def init(table, reload=False):
 
-    if 'xray' in table.properties and not reload: return
+    if 'xray' in table.properties and not reload:
+        return
     table.properties.append('xray')
 
     # Create an xray object for the particular element/ion.  Note that
-    # we must not use normal attribute tests such as "hasattr(el,'attr')"
+    # we must not use normal attribute tests such as "hasattr(el, 'attr')"
     # or "try: el.attr; except:" since the delegation methods on Ion will
     # just return the attribute from the base element.  Instead we check
     # for an instance specific xray object for the particular ion prior
@@ -636,7 +720,7 @@ def init(table, reload=False):
     # TODO: is there a better way to set up delegation on a field by
     # field basis?
     def _cache_xray(el):
-        if '_xray' not in el.__dict__ and isinstance(el, (Element,Ion)):
+        if '_xray' not in el.__dict__ and isinstance(el, (Element, Ion)):
             el._xray = Xray(el)
         return el._xray
     Element.xray = property(_cache_xray)
@@ -657,15 +741,16 @@ def plot_xsf(el):
 
     :Returns: None
     """
-    import pylab
+    import matplotlib.pyplot as plt
+
     xsf = el.xray.sftable
-    pylab.title('X-ray scattering factors for '+el.name)
-    pylab.plot(xsf[0],xsf[1])
-    pylab.plot(xsf[0],xsf[2])
-    pylab.xlabel('Energy (keV)')
-    pylab.ylabel('Scattering factor')
-    pylab.legend(['f1','f2'])
-    pylab.show()
+    plt.title('X-ray scattering factors for '+el.name)
+    plt.plot(xsf[0], xsf[1])
+    plt.plot(xsf[0], xsf[2])
+    plt.xlabel('Energy (keV)')
+    plt.ylabel('Scattering factor')
+    plt.legend(['f1', 'f2'])
+    plt.show()
 
 def sld_table(wavelength=None, table=None):
     """
@@ -689,7 +774,7 @@ def sld_table(wavelength=None, table=None):
          Li   3.92   0.00
          Be  13.93   0.01
           B  18.40   0.01
-          C  17.86   0.03
+          C  18.71   0.03
           N   6.88   0.02
           O   9.74   0.04
           F  12.16   0.07
@@ -699,15 +784,16 @@ def sld_table(wavelength=None, table=None):
           ...
     """
     table = default_table(table)
-    if wavelength == None: wavelength = table.Cu.K_alpha
+    if wavelength is None:
+        wavelength = table.Cu.K_alpha
 
     # NBCU spreadsheet format
     print("X-ray scattering length density for %g Ang"%wavelength)
-    print("%3s %6s %6s"%('El','rho','irho'))
+    print("%3s %6s %6s"%('El', 'rho', 'irho'))
     for el in table:
-        rho,irho = el.xray.sld(wavelength=table.Cu.K_alpha)
+        rho, irho = el.xray.sld(wavelength=table.Cu.K_alpha)
         if rho is not None:
-            print("%3s %6.2f %6.2f"%(el.symbol,rho,irho))
+            print("%3s %6.2f %6.2f"%(el.symbol, rho, irho))
 
 def emission_table(table=None):
     """
@@ -731,8 +817,7 @@ def emission_table(table=None):
          ...
     """
     table = default_table(table)
-    print("%3s %7s %7s"%('El','Kalpha','Kbeta1'))
+    print("%3s %7s %7s"%('El', 'Kalpha', 'Kbeta1'))
     for el in table:
-        if hasattr(el,'K_alpha'):
-            print("%3s %7.4f %7.4f"%(el.symbol,el.K_alpha,el.K_beta1))
-
+        if hasattr(el, 'K_alpha'):
+            print("%3s %7.4f %7.4f"%(el.symbol, el.K_alpha, el.K_beta1))
